#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import math

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.features.search import ISearchFolder
from pyams_content.features.search.portlet.interfaces import ISearchResultsPortletSettings
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import VIEW_PERMISSION
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent

from pyams_content import _


SEARCH_RESULTS_PORTLET_NAME = 'pyams_content.portlet.search.results'


@implementer(ISearchResultsPortletSettings)
@factory_config(provided=ISearchResultsPortletSettings)
class SearchResultsPortletSettings(PortletSettings):
    """Search results portlet settings"""

    title = FieldProperty(ISearchResultsPortletSettings['title'])

    @staticmethod
    def get_items(request=None, limit=None, ignore_cache=False):
        context = get_parent(request.context, ISearchFolder)
        if context is None:
            raise StopIteration
        if request is None:
            request = check_request()
        params = request.params
        sort_index = params.get('order_by', RELEVANCE_ORDER)
        yield from context.get_results(context, sort_index,
                                       reverse=sort_index != RELEVANCE_ORDER,
                                       limit=limit,
                                       start=int(params.get('start', 0)),
                                       length=int(params.get('length', 10)),
                                       ignore_cache=ignore_cache,
                                       get_count=True)

    @staticmethod
    def get_pages(request, count):
        params = request.params
        start = int(params.get('start', 0)) + 1
        length = int(params.get('length', 10))
        current = math.ceil(start / length)
        nb_pages = math.ceil(count / length)
        return current, nb_pages


@portlet_config(permission=VIEW_PERMISSION)
class SearchResultsPortlet(Portlet):
    """Search results portlet"""

    name = SEARCH_RESULTS_PORTLET_NAME
    label = _("Search results")

    toolbar_css_class = 'fa fa-fw fa-2x fa-search-plus'

    settings_factory = ISearchResultsPortletSettings
